package org.jboss.hibernate2.jmx;

import org.jboss.deployment.DeploymentInfo;

import java.net.URL;
import java.util.Date;

import net.sf.hibernate.SessionFactory;

import javax.management.ObjectName;

/**
 * Describes a Hibernate service MBean.  Configures a
 * {@link net.sf.hibernate.SessionFactory} instance and exposes it through JNDI.
 * The SessionFactory is built through either<ul>
 * <li>auto-discovery : where the classpath of the top-level deployment
 * containing this MBean is searched for jars and directories to add to the
 * config
 * <li>har-deployment : where the deployment containing this MBean description
 * conforms to the har layout definition.
 * </ul>
 */
public interface HibernateMBean extends org.jboss.system.ServiceMBean, javax.management.MBeanRegistration {

	public static final javax.management.ObjectName OBJECT_NAME = org.jboss.mx.util.ObjectNameFactory.create(
			"jboss.har:service=Hibernate"
	);


	/**
    * The url to the har deployment, if MBean is operating in har deployment mode.
	 *
	 * @return The url of the har containing this MBean, or null if not part of
	 * a har deployment.
	 */
	public URL getHarUrl();

	/**
	 * Enables scanning of the entire deployment classpath for any potential mapping
	 * sources (jars or directories).
	 * <p/>
	 * Only used in the case of har deployments.
	 *
	 * @return should we aggressively scan for addition mappings not
	 * strictly following the HAR-format?
	 */
	public boolean isScanForMappingsEnabled();

	/**
	 * Enables scanning of the entire deployment classpath for any potential mapping
	 * sources (jars or directories).
	 * <p/>
	 * Only used in the case of har deployments.
	 *
	 * @param scanForMappings
	 */
	public void setScanForMappingsEnabled(boolean scanForMappings);

   /**
    * Retrieve MBean's current deployment info.
    *
    * @return the deployment info.
    */
   public DeploymentInfo getDeploymentInfo();

	/**
	 * The JNDI namespace where the managed {@link net.sf.hibernate.SessionFactory} is to be bound.
	 *
	 * @return The current setting value.
	 */
	public String getSessionFactoryName();

	/**
	 * The JNDI namespace where the managed {@link net.sf.hibernate.SessionFactory} is to be bound.
	 *
	 * @param sessionFactoryName The new JNDI namespace to use.
	 */
	public void setSessionFactoryName(String sessionFactoryName);

	/**
	 * The JMX name of a {@link org.jboss.cache.TreeCache} MBean to be used as the second level cache.
	 * <p/>
	 * Note : only used when {@link #getCacheProviderClass} == {@link org.jboss.hibernate2.cache.DeployedTreeCacheProvider}
	 *
	 * @return The current setting
	 */
	public ObjectName getDeployedTreeCacheObjectName();

	/**
	 * The JMX name of a {@link org.jboss.cache.TreeCache} MBean to be used as the second level cache.
	 *
	 * @param deployedTreeCacheObjectName The new mbean object name.
	 */
	public void setDeployedTreeCacheObjectName(ObjectName deployedTreeCacheObjectName);

	/**
	 * The name of the dialect class to use for communicating with the database.
	 *
	 * @return The current setting value.
	 *
	 * @see net.sf.hibernate.cfg.Environment#DIALECT
	 */
	public String getDialect();

	/**
	 * The name of the dialect class to use for communicating with the database.
	 *
	 * @param dialect The new dialect class name to use.
	 */
	public void setDialect(String dialect);

	/**
	 * The form, if any, of schema generation which should be used.
	 *
	 * @return The current setting value.
	 *
	 * @see net.sf.hibernate.cfg.Environment#HBM2DDL_AUTO
	 */
	public String getHbm2ddlAuto();

	/**
	 * The form, if any, of schema generation which should be used.
	 *
	 * @param hbm2ddlAuto The new hbm2ddl setting; valid values are: update, create, create-drop
	 */
	public void setHbm2ddlAuto(String hbm2ddlAuto);

	/**
	 * The JNDI namespace of the {@link javax.sql.DataSource} which should be used by the managed {@link
	 * net.sf.hibernate.SessionFactory}.
	 *
	 * @return The current setting value.
	 *
	 * @see net.sf.hibernate.cfg.Environment#DATASOURCE
	 */
	public String getDatasourceName();

	/**
	 * The JNDI namespace of the {@link javax.sql.DataSource} which should be used by the managed {@link
	 * net.sf.hibernate.SessionFactory}.
	 *
	 * @param datasourceName The new DataSource JNDI name to use.
	 */
	public void setDatasourceName(String datasourceName);

	/**
	 * The username used to access the specified datasource.
	 *
	 * @return The current setting value.
	 *
	 * @see net.sf.hibernate.cfg.Environment#USER
	 */
	public String getUsername();

	/**
	 * The username used to access the specified datasource.
	 *
	 * @param username The new username value.
	 */
	public void setUsername(String username);

	/**
	 * The password used to access the specified datasource.
	 *
	 * @param password The new password value.
	 */
	public void setPassword(String password);

	/**
	 * The default database schema to use within the database being mapped.
	 * <p/>
	 * Used for databases which support the concept of schemas instead of catalogs.
	 *
	 * @return The current setting value.
	 *
	 * @see net.sf.hibernate.cfg.Environment#DEFAULT_SCHEMA
	 */
	public String getDefaultSchema();

	/**
	 * The default database schema to use within the database being mapped.
	 *
	 * @param defaultSchema The new default schema name to use.
	 */
	public void setDefaultSchema(String defaultSchema);

	/**
	 * The maximum outer join fetch depth.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#MAX_FETCH_DEPTH
	 */
	public Integer getMaxFetchDepth();

	/**
	 * The maximum outer join fetch depth.
	 *
	 * @param maxFetchDepth The new max fetch depth value
	 */
	public void setMaxFetchDepth(Integer maxFetchDepth);

	/**
	 * The JDBC batch update batch size.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#STATEMENT_BATCH_SIZE
	 */
	public Integer getJdbcBatchSize();

	/**
	 * The JDBC batch update batch size.
	 *
	 * @param jdbcBatchSize The new value for the number of statements to batch together.
	 */
	public void setJdbcBatchSize(Integer jdbcBatchSize);

	/**
	 * The JDBC fetch size.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#STATEMENT_FETCH_SIZE
	 */
	public Integer getJdbcFetchSize();

	/**
	 * The JDBC fetch size.
	 *
	 * @param jdbcFetchSize The new value for the number of rows to fetch from server at a time.
	 */
	public void setJdbcFetchSize(Integer jdbcFetchSize);

	/**
	 * Are scrollable result sets enabled?
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#USE_SCROLLABLE_RESULTSET
	 */
	public Boolean getJdbcScrollableResultSetEnabled();

	/**
	 * Are scrollable result sets enabled?
	 *
	 * @param jdbcScrollableResultSetEnabled The new value.
	 */
	public void setJdbcScrollableResultSetEnabled(Boolean jdbcScrollableResultSetEnabled);

	/**
	 * Is the use of JDBC3 <tt>getGeneratedKeys()</tt> enabled?
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#USE_GET_GENERATED_KEYS
	 */
	public Boolean getGetGeneratedKeysEnabled();

	/**
	 * Is the use of JDBC3 <tt>getGeneratedKeys()</tt> enabled?
	 *
	 * @param getGeneratedKeysEnabled The new value.
	 */
	public void setGetGeneratedKeysEnabled(Boolean getGeneratedKeysEnabled);

	/**
	 * Should Hibernate allow JDBC batch-updating of versioned entities?
	 * <p/>
	 * Many drivers have bugs regarding the row counts returned in response to JDBC Batch API operations; in these cases,
	 * this should definitely be set to false.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#BATCH_VERSIONED_DATA
	 */
	public Boolean getBatchVersionedDataEnabled();

	/**
	 * Should Hibernate allow JDBC batch-updating of versioned entities?
	 *
	 * @param batchVersionedDataEnabled
	 */
	public void setBatchVersionedDataEnabled(Boolean batchVersionedDataEnabled);

	/**
	 * Query substitutions to use.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#QUERY_SUBSTITUTIONS
	 */
	public String getQuerySubstitutions();

	/**
	 * Query substitutions to use.
	 *
	 * @param querySubstitutions The new query substitutions to use
	 */
	public void setQuerySubstitutions(String querySubstitutions);

	/**
	 * The name of the {@link net.sf.hibernate.cache.CacheProvider} implementation class to use for second level caching.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#CACHE_PROVIDER
	 */
	public String getCacheProviderClass();

	/**
	 * The name of the {@link net.sf.hibernate.cache.CacheProvider} implementation class to use for second level caching.
	 *
	 * @param cacheProviderClass The new provider impl class name.
	 */
	public void setCacheProviderClass(String cacheProviderClass);

	/**
	 * The prefix to use for this session factory within the second level cache.
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#CACHE_REGION_PREFIX
	 */
	public String getCacheRegionPrefix();

	/**
	 * The prefix to use for this session factory within the second level cache.
	 *
	 * @param cacheRegionPrefix The new prefix value.
	 */
	public void setCacheRegionPrefix(String cacheRegionPrefix);

	/**
	 * Should minimal puts be enabled against the given cache provider?
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#USE_MINIMAL_PUTS
	 */
	public Boolean getMinimalPutsEnabled();

	/**
	 * Should minimal puts be enabled against the given cache provider?
	 *
	 * @param minimalPutsEnabled
	 */
	public void setMinimalPutsEnabled(Boolean minimalPutsEnabled);

	/**
	 * Is use of the query cache enabled?
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#USE_QUERY_CACHE
	 */
	public Boolean getQueryCacheEnabled();

	/**
	 * Is use of the query cache enabled?
	 *
	 * @param queryCacheEnabled The new value of whether or not to enable.
	 */
	public void setQueryCacheEnabled(Boolean queryCacheEnabled);

	/**
	 * Should all SQL be shown (dumped to console and logged)?
	 *
	 * @return The current setting value
	 *
	 * @see net.sf.hibernate.cfg.Environment#SHOW_SQL
	 */
	public Boolean getShowSqlEnabled();

	/**
	 * Should all SQL be shown (dumped to console and logged)?
	 *
	 * @param showSqlEnabled
	 */
	public void setShowSqlEnabled(Boolean showSqlEnabled);

	/**
	 * The name of an {@link net.sf.hibernate.Interceptor} impl class to be attached to the managed {@link
	 * net.sf.hibernate.SessionFactory}.
	 *
	 * @return Name of the Interceptor class
	 */
	public String getSessionFactoryInterceptor();

	/**
	 * The name of an {@link net.sf.hibernate.Interceptor} impl class to be attached to the managed {@link
	 * net.sf.hibernate.SessionFactory}.
	 *
	 * @param sessionFactoryInterceptor
	 */
	public void setSessionFactoryInterceptor(String sessionFactoryInterceptor);

	/**
	 * Is this MBean dirty?  Meaning, have any changes been made to it that have not yet been propogated to the managed
	 * {@link net.sf.hibernate.SessionFactory}?
	 * <p/>
	 * Note : the only way to propogate these changes to the SF is by calling the {@link #rebuildSessionFactory()} managed
	 * operation.
	 */
	public boolean isDirty();

	/**
	 * Does this MBean instance have a currently running managed {@link net.sf.hibernate.SessionFactory}?
	 */
	public boolean isSessionFactoryRunning();

	/**
	 * The version Hibernate for the managed {@link net.sf.hibernate.SessionFactory}.
	 */
	public String getVersion();

	/**
	 * Exposes the internally managed session factory via a read-only JMX managed attribute.
	 *
	 * @return The managed session factory.
	 */
	public SessionFactory getInstance();

   /**
    * The date and time since which the currently managed {@link net.sf.hibernate.SessionFactory} has been running.
    *
    * @return The date and time the current {@link net.sf.hibernate.SessionFactory} was started.
    */
   public Date getRunningSince();

	/**
	 * A JMX managed operation to rebuild the managed {@link net.sf.hibernate.SessionFactory} such that any setting changes
	 * made can take effect.
	 *
	 * @throws Exception
	 */
	public void rebuildSessionFactory() throws Exception;

}
