/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.hibernate2.session;

import net.sf.hibernate.Session;
import org.jboss.logging.Logger;
import org.jboss.util.NestedRuntimeException;

import javax.transaction.Synchronization;

/**
 * Implementation of a transaction Synchronization for managing closure
 * of a hibernate Session on completion of the JTA transaction.
 */
class TransactionSynch implements Synchronization
{
   private static Logger log = Logger.getLogger(TransactionSynch.class);

   private String name;
   private Session session;

   public TransactionSynch(String name, Session session)
   {
      if ( log.isTraceEnabled() )
      {
         log.trace("Preparing synch [name=" + name + ", session=" + session + "]");
      }

      if (session == null)
      {
         throw new IllegalArgumentException("Session to synchronize cannot be null");
      }

      this.name = name;
      this.session = session;
   }

   public void beforeCompletion()
   {
      if ( !session.isOpen() )
      {
         log.warn("Session already closed");
      }
      else
      {
         try
         {
            log.trace("Flushing Session");
            session.flush();
         }
         catch(Throwable t)
         {
            throw new FlushException( "Error flushing session", t );
         }
      }
   }

   public void afterCompletion(int status)
   {
      if ( session.isOpen() )
      {
         try
         {
            log.trace("Closing Session");
            session.close();
         }
         catch(Throwable t)
         {
            log.warn("Unable to close session");
         }
      }

      if ( session != HibernateContext.unbind(name) )
      {
         log.warn("Synch encountered different session in context while performing unbind");
      }
   }

   public static class FlushException extends NestedRuntimeException
   {
      public FlushException(String msg)
      {
         super( msg );
      }

      public FlushException(String msg, Throwable nested)
      {
         super( msg, nested );
      }
   }
}
